/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.content.Context;
import android.database.Cursor;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Data;
import android.telephony.PhoneNumberUtils;
import android.util.Log;

import java.util.HashMap;

public class Contact {
    // 静态缓存，用于存储已查询过的联系人信息，避免重复查询数据库
    private static HashMap<String, String> sContactCache;
    // 日志标签，用于调试和记录日志信息
    private static final String TAG = "Contact";

    // 查询联系人的SQL语句模板
    // 该语句用于根据电话号码查询联系人名称
    // PHONE_NUMBERS_EQUAL 是一个辅助函数，用于比较电话号码是否匹配
    // Data.MIMETYPE 用于指定查询的数据类型为电话号码
    // Data.RAW_CONTACT_ID 用于关联 phone_lookup 表，确保查询结果的准确性
    private static final String CALLER_ID_SELECTION = "PHONE_NUMBERS_EQUAL(" + Phone.NUMBER
    + ",?) AND " + Data.MIMETYPE + "='" + Phone.CONTENT_ITEM_TYPE + "'"
    + " AND " + Data.RAW_CONTACT_ID + " IN "
            + "(SELECT raw_contact_id "
            + " FROM phone_lookup"
            + " WHERE min_match = '+')";

    /**
     * 根据电话号码获取联系人名称
     *
     * @param context   应用程序上下文
     * @param phoneNumber 要查询的电话号码
     * @return 联系人名称，如果未找到则返回 null
     */
    public static String getContact(Context context, String phoneNumber) {
        // 如果缓存为空，则初始化缓存
        if(sContactCache == null) {
            sContactCache = new HashMap<String, String>();
        }

        // 检查缓存中是否已经存在该电话号码的联系人信息
        if(sContactCache.containsKey(phoneNumber)) {
            // 如果缓存中有，直接返回缓存中的联系人名称
            return sContactCache.get(phoneNumber);
        }

        // 替换 SQL 查询语句中的占位符为实际的电话号码
        // PhoneNumberUtils.toCallerIDMinMatch 用于将电话号码转换为适合查询的格式
        String selection = CALLER_ID_SELECTION.replace("+",
                PhoneNumberUtils.toCallerIDMinMatch(phoneNumber));

        // 查询联系人数据库
        Cursor cursor = context.getContentResolver().query(
                Data.CONTENT_URI, // 查询的 URI
                new String [] { Phone.DISPLAY_NAME }, // 查询返回的列，这里只需要联系人名称
                selection, // 查询条件
                new String[] { phoneNumber }, // 查询参数
                null); // 不需要排序

        // 检查查询结果
        if (cursor != null && cursor.moveToFirst()) {
            try {
                // 获取查询结果中的联系人名称
                String name = cursor.getString(0);
                // 将查询结果存入缓存
                sContactCache.put(phoneNumber, name);
                // 返回联系人名称
                return name;
            } catch (IndexOutOfBoundsException e) {
                // 如果发生异常，记录错误日志
                Log.e(TAG, " Cursor get string error " + e.toString());
                return null;
            } finally {
                // 关闭游标，释放资源
                cursor.close();
            }
        } else {
            // 如果没有找到匹配的联系人，记录调试日志
            Log.d(TAG, "No contact matched with number:" + phoneNumber);
            return null;
        }
    }
}