/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * MetaData 类用于管理与 Google Tasks 同步相关的元数据
 * 该类继承自 Task 类，主要用于存储和处理与 Google Tasks 相关的任务 ID 和同步信息
 * 元数据以 JSON 格式存储在笔记内容中
 */
public class MetaData extends Task {
    private final static String TAG = MetaData.class.getSimpleName();

    // 关联的 Google Task ID
    private String mRelatedGid = null;

    /**
     * 设置元数据信息
     * 
     * @param gid Google Task ID
     * @param metaInfo 元数据 JSON 对象
     */
    public void setMeta(String gid, JSONObject metaInfo) {
        try {
            // 将 Google Task ID 添加到元数据中
            metaInfo.put(GTaskStringUtils.META_HEAD_GTASK_ID, gid);
        } catch (JSONException e) {
            Log.e(TAG, "failed to put related gid");
        }
        // 将元数据转换为字符串并存储在笔记内容中
        setNotes(metaInfo.toString());
        // 设置固定的元数据笔记名称
        setName(GTaskStringUtils.META_NOTE_NAME);
    }

    /**
     * 获取关联的 Google Task ID
     * 
     * @return Google Task ID
     */
    public String getRelatedGid() {
        return mRelatedGid;
    }

    /**
     * 判断元数据是否值得保存
     * 
     * @return 如果笔记内容不为空则返回 true，否则返回 false
     */
    @Override
    public boolean isWorthSaving() {
        return getNotes() != null;
    }

    /**
     * 从远程 JSON 数据设置内容
     * 
     * @param js 远程 JSON 对象
     */
    @Override
    public void setContentByRemoteJSON(JSONObject js) {
        // 调用父类方法设置基本内容
        super.setContentByRemoteJSON(js);
        if (getNotes() != null) {
            try {
                // 解析笔记内容中的 JSON 元数据
                JSONObject metaInfo = new JSONObject(getNotes().trim());
                // 提取并存储关联的 Google Task ID
                mRelatedGid = metaInfo.getString(GTaskStringUtils.META_HEAD_GTASK_ID);
            } catch (JSONException e) {
                Log.w(TAG, "failed to get related gid");
                mRelatedGid = null;
            }
        }
    }

    /**
     * 从本地 JSON 设置内容 - 此方法不应该被调用
     * MetaData 不应该从本地 JSON 设置内容，因此抛出异常
     */
    @Override
    public void setContentByLocalJSON(JSONObject js) {
        // this function should not be called
        throw new IllegalAccessError("MetaData:setContentByLocalJSON should not be called");
    }

    /**
     * 从内容获取本地 JSON - 此方法不应该被调用
     * MetaData 不应该提供本地 JSON 转换，因此抛出异常
     */
    @Override
    public JSONObject getLocalJSONFromContent() {
        throw new IllegalAccessError("MetaData:getLocalJSONFromContent should not be called");
    }

    /**
     * 获取同步操作 - 此方法不应该被调用
     * MetaData 不参与常规同步操作判断，因此抛出异常
     */
    @Override
    public int getSyncAction(Cursor c) {
        throw new IllegalAccessError("MetaData:getSyncAction should not be called");
    }
}