/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;

import org.json.JSONObject;

/**
 * Node 类是 Google Tasks 同步框架中的抽象基类
 * 定义了所有可同步数据节点的通用接口和属性
 * 主要用于处理本地笔记与 Google Tasks 之间的数据同步
 */
public abstract class Node {
    // 同步动作类型常量定义
    public static final int SYNC_ACTION_NONE = 0;           // 无需同步
    
    public static final int SYNC_ACTION_ADD_REMOTE = 1;     // 添加到远程
    public static final int SYNC_ACTION_ADD_LOCAL = 2;      // 添加到本地
    
    public static final int SYNC_ACTION_DEL_REMOTE = 3;     // 从远程删除
    public static final int SYNC_ACTION_DEL_LOCAL = 4;      // 从本地删除
    
    public static final int SYNC_ACTION_UPDATE_REMOTE = 5;  // 更新远程数据
    public static final int SYNC_ACTION_UPDATE_LOCAL = 6;   // 更新本地数据
    
    public static final int SYNC_ACTION_UPDATE_CONFLICT = 7; // 同步冲突
    public static final int SYNC_ACTION_ERROR = 8;           // 同步错误

    // 节点基本属性
    private String mGid;           // Google Task ID，用于标识远程任务
    private String mName;          // 节点名称
    private long mLastModified;    // 最后修改时间戳
    private boolean mDeleted;      // 删除标记

    /**
     * 默认构造函数，初始化节点属性
     */
    public Node() {
        mGid = null;
        mName = "";
        mLastModified = 0;
        mDeleted = false;
    }

    /**
     * 创建节点创建操作的 JSON 描述
     * @param actionId 操作ID
     * @return 包含创建操作信息的 JSON 对象
     */
    public abstract JSONObject getCreateAction(int actionId);

    /**
     * 创建节点更新操作的 JSON 描述
     * @param actionId 操作ID
     * @return 包含更新操作信息的 JSON 对象
     */
    public abstract JSONObject getUpdateAction(int actionId);

    /**
     * 从远程 JSON 数据设置节点内容
     * @param js 包含远程数据的 JSON 对象
     */
    public abstract void setContentByRemoteJSON(JSONObject js);

    /**
     * 从本地 JSON 数据设置节点内容
     * @param js 包含本地数据的 JSON 对象
     */
    public abstract void setContentByLocalJSON(JSONObject js);

    /**
     * 将节点内容转换为本地 JSON 格式
     * @return 包含节点内容的 JSON 对象
     */
    public abstract JSONObject getLocalJSONFromContent();

    /**
     * 根据本地数据库游标确定同步操作类型
     * @param c 数据库游标
     * @return 同步操作类型常量
     */
    public abstract int getSyncAction(Cursor c);

    // 基本属性的 getter 和 setter 方法
    public void setGid(String gid) {
        this.mGid = gid;
    }

    public void setName(String name) {
        this.mName = name;
    }

    public void setLastModified(long lastModified) {
        this.mLastModified = lastModified;
    }

    public void setDeleted(boolean deleted) {
        this.mDeleted = deleted;
    }

    public String getGid() {
        return this.mGid;
    }

    public String getName() {
        return this.mName;
    }

    public long getLastModified() {
        return this.mLastModified;
    }

    public boolean getDeleted() {
        return this.mDeleted;
    }
}