/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;

import net.micode.notes.R;
import net.micode.notes.ui.NotesListActivity;
import net.micode.notes.ui.NotesPreferenceActivity;

/**
 * 该类继承自 AsyncTask，用于在后台执行 Google 任务同步操作。
 * 同步过程中会显示通知，同步完成后会根据结果显示不同的通知信息。
 */
public class GTaskASyncTask extends AsyncTask<Void, String, Integer> {

    // 同步通知的唯一 ID
    private static int GTASK_SYNC_NOTIFICATION_ID = 5234235;

    /**
     * 同步完成监听器接口，用于在同步完成后执行特定操作
     */
    public interface OnCompleteListener {
        void onComplete();
    }

    // 应用上下文
    private Context mContext;
    // 通知管理器
    private NotificationManager mNotifiManager;
    // GTask 管理器实例
    private GTaskManager mTaskManager;
    // 同步完成监听器
    private OnCompleteListener mOnCompleteListener;

    /**
     * 构造函数，初始化上下文、监听器、通知管理器和 GTask 管理器
     * @param context 应用上下文
     * @param listener 同步完成监听器
     */
    public GTaskASyncTask(Context context, OnCompleteListener listener) {
        mContext = context;
        mOnCompleteListener = listener;
        mNotifiManager = (NotificationManager) mContext
                .getSystemService(Context.NOTIFICATION_SERVICE);
        mTaskManager = GTaskManager.getInstance();
    }

    /**
     * 取消同步操作
     */
    public void cancelSync() {
        mTaskManager.cancelSync();
    }

    /**
     * 发布同步进度信息
     * @param message 进度信息
     */
    public void publishProgess(String message) {
        publishProgress(new String[]{
                message
        });
    }

    /**
     * 显示同步通知
     * @param tickerId 通知的提示信息资源 ID
     * @param content 通知的内容
     */
    private void showNotification(int tickerId, String content) {
        PendingIntent pendingIntent;
        // 根据不同的提示信息 ID 设置不同的点击跳转意图
        if (tickerId != R.string.ticker_success) {
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext,
                    NotesPreferenceActivity.class), PendingIntent.FLAG_IMMUTABLE);
        } else {
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext,
                    NotesListActivity.class), PendingIntent.FLAG_IMMUTABLE);
        }
        Notification.Builder builder = new Notification.Builder(mContext)
                .setAutoCancel(true)
                .setContentTitle(mContext.getString(R.string.app_name))
                .setContentText(content)
                .setContentIntent(pendingIntent)
                .setWhen(System.currentTimeMillis())
                .setOngoing(true);
        Notification notification = builder.getNotification();
        mNotifiManager.notify(GTASK_SYNC_NOTIFICATION_ID, notification);
    }

    /**
     * 后台执行同步操作
     * @param unused 未使用的参数
     * @return 同步结果状态码
     */
    @Override
    protected Integer doInBackground(Void... unused) {
        // 发布登录进度信息
        publishProgess(mContext.getString(R.string.sync_progress_login, NotesPreferenceActivity
                .getSyncAccountName(mContext)));
        // 调用 GTask 管理器的同步方法
        return mTaskManager.sync(mContext, this);
    }

    /**
     * 更新同步进度，显示进度通知并发送广播
     * @param progress 进度信息
     */
    @Override
    protected void onProgressUpdate(String... progress) {
        showNotification(R.string.ticker_syncing, progress[0]);
        if (mContext instanceof GTaskSyncService) {
            ((GTaskSyncService) mContext).sendBroadcast(progress[0]);
        }
    }

    /**
     * 同步完成后执行的操作，根据同步结果显示不同的通知信息
     * @param result 同步结果状态码
     */
    @Override
    protected void onPostExecute(Integer result) {
        if (result == GTaskManager.STATE_SUCCESS) {
            showNotification(R.string.ticker_success, mContext.getString(
                    R.string.success_sync_account, mTaskManager.getSyncAccount()));
            // 设置最后同步时间
            NotesPreferenceActivity.setLastSyncTime(mContext, System.currentTimeMillis());
        } else if (result == GTaskManager.STATE_NETWORK_ERROR) {
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_network));
        } else if (result == GTaskManager.STATE_INTERNAL_ERROR) {
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_internal));
        } else if (result == GTaskManager.STATE_SYNC_CANCELLED) {
            showNotification(R.string.ticker_cancel, mContext
                    .getString(R.string.error_sync_cancelled));
        }
        if (mOnCompleteListener != null) {
            // 启动新线程调用同步完成监听器的方法
            new Thread(new Runnable() {
                public void run() {
                    mOnCompleteListener.onComplete();
                }
            }).start();
        }
    }
}