/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote;

import android.app.Activity;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.IBinder;

// GTaskSyncService 是一个用于管理 Google 任务同步的服务
public class GTaskSyncService extends Service {
    // 定义广播相关的常量
    public final static String ACTION_STRING_NAME = "sync_action_type"; // 广播中用于标识同步操作类型的键
    public final static int ACTION_START_SYNC = 0; // 开始同步操作
    public final static int ACTION_CANCEL_SYNC = 1; // 取消同步操作
    public final static int ACTION_INVALID = 2; // 无效操作

    public final static String GTASK_SERVICE_BROADCAST_NAME = "net.micode.notes.gtask.remote.gtask_sync_service"; // 广播的名称
    public final static String GTASK_SERVICE_BROADCAST_IS_SYNCING = "isSyncing"; // 广播中用于标识是否正在同步的键
    public final static String GTASK_SERVICE_BROADCAST_PROGRESS_MSG = "progressMsg"; // 广播中用于标识同步进度信息的键

    // 静态变量，用于管理同步任务
    private static GTaskASyncTask mSyncTask = null; // 当前的同步任务
    private static String mSyncProgress = ""; // 同步进度信息

    // 开始同步任务
    private void startSync() {
        if (mSyncTask == null) {
            mSyncTask = new GTaskASyncTask(this, new GTaskASyncTask.OnCompleteListener() {
                public void onComplete() {
                    mSyncTask = null; // 同步完成时，清空同步任务
                    sendBroadcast(""); // 发送广播通知同步完成
                    stopSelf(); // 停止服务
                }
            });
            sendBroadcast(""); // 发送广播通知同步开始
            mSyncTask.execute(); // 执行同步任务
        }
    }

    // 取消同步任务
    private void cancelSync() {
        if (mSyncTask != null) {
            mSyncTask.cancelSync(); // 调用同步任务的取消方法
        }
    }

    // 服务创建时调用
    @Override
    public void onCreate() {
        mSyncTask = null; // 初始化同步任务为 null
    }

    // 服务启动时调用
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Bundle bundle = intent.getExtras(); // 获取启动服务时传递的额外数据
        if (bundle != null && bundle.containsKey(ACTION_STRING_NAME)) {
            switch (bundle.getInt(ACTION_STRING_NAME, ACTION_INVALID)) {
                case ACTION_START_SYNC: // 如果是开始同步操作
                    startSync(); // 调用 startSync 方法开始同步
                    break;
                case ACTION_CANCEL_SYNC: // 如果是取消同步操作
                    cancelSync(); // 调用 cancelSync 方法取消同步
                    break;
                default: // 其他无效操作
                    break;
            }
            return START_STICKY; // 返回 START_STICKY，表示如果服务被系统杀死，系统会尝试重新启动服务
        }
        return super.onStartCommand(intent, flags, startId); // 如果没有传递有效的操作类型，调用父类方法
    }

    // 在低内存情况下调用
    @Override
    public void onLowMemory() {
        if (mSyncTask != null) {
            mSyncTask.cancelSync(); // 在低内存情况下取消同步任务
        }
    }

    // 服务绑定时调用
    public IBinder onBind(Intent intent) {
        return null; // 该服务不支持绑定，返回 null
    }

    // 发送广播通知同步状态或进度
    public void sendBroadcast(String msg) {
        mSyncProgress = msg; // 更新同步进度信息
        Intent intent = new Intent(GTASK_SERVICE_BROADCAST_NAME); // 创建广播意图
        intent.putExtra(GTASK_SERVICE_BROADCAST_IS_SYNCING, mSyncTask != null); // 添加是否正在同步的信息
        intent.putExtra(GTASK_SERVICE_BROADCAST_PROGRESS_MSG, msg); // 添加同步进度信息
        sendBroadcast(intent); // 发送广播
    }

    // 静态方法，用于从 Activity 启动同步服务
    public static void startSync(Activity activity) {
        GTaskManager.getInstance().setActivityContext(activity); // 设置 Activity 上下文到 GTaskManager
        Intent intent = new Intent(activity, GTaskSyncService.class); // 创建启动服务的意图
        intent.putExtra(GTASK_SERVICE_BROADCAST_NAME, ACTION_START_SYNC); // 添加开始同步的操作类型
        activity.startService(intent); // 启动服务
    }

    // 静态方法，用于取消同步
    public static void cancelSync(Context context) {
        Intent intent = new Intent(context, GTaskSyncService.class); // 创建启动服务的意图
        intent.putExtra(GTASK_SERVICE_BROADCAST_NAME, ACTION_CANCEL_SYNC); // 添加取消同步的操作类型
        context.startService(intent); // 启动服务
    }

    // 静态方法，用于检查是否正在同步
    public static boolean isSyncing() {
        return mSyncTask != null; // 如果 mSyncTask 不为 null，则表示正在同步
    }

    // 静态方法，用于获取同步进度信息
    public static String getProgressString() {
        return mSyncProgress; // 返回同步进度信息
    }
}