/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.model;

import android.appwidget.AppWidgetManager;
import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;
import net.micode.notes.tool.ResourceParser.NoteBgResources;

/**
 * WorkingNote 类用于管理和操作笔记的相关信息，包括笔记的基本属性、内容、模式等，
 * 同时提供了创建、加载、保存笔记等功能。
 */
public class WorkingNote {
    // 笔记对象
    private Note mNote;
    // 笔记的 ID
    private long mNoteId;
    // 笔记的内容
    private String mContent;
    // 笔记的模式
    private int mMode;
    // 笔记的提醒日期
    private long mAlertDate;
    // 笔记的修改日期
    private long mModifiedDate;
    // 笔记的背景颜色 ID
    private int mBgColorId;
    // 笔记关联的小部件 ID
    private int mWidgetId;
    // 笔记关联的小部件类型
    private int mWidgetType;
    // 笔记所在的文件夹 ID
    private long mFolderId;
    // 上下文对象
    private Context mContext;
    // 日志标签
    private static final String TAG = "WorkingNote";
    // 笔记是否被删除的标志
    private boolean mIsDeleted;
    // 笔记设置变化监听器
    private NoteSettingChangedListener mNoteSettingStatusListener;

    // 数据查询的投影列
    public static final String[] DATA_PROJECTION = new String[] {
            DataColumns.ID,
            DataColumns.CONTENT,
            DataColumns.MIME_TYPE,
            DataColumns.DATA1,
            DataColumns.DATA2,
            DataColumns.DATA3,
            DataColumns.DATA4,
    };

    // 笔记查询的投影列
    public static final String[] NOTE_PROJECTION = new String[] {
            NoteColumns.PARENT_ID,
            NoteColumns.ALERTED_DATE,
            NoteColumns.BG_COLOR_ID,
            NoteColumns.WIDGET_ID,
            NoteColumns.WIDGET_TYPE,
            NoteColumns.MODIFIED_DATE
    };

    // 数据 ID 列的索引
    private static final int DATA_ID_COLUMN = 0;
    // 数据内容列的索引
    private static final int DATA_CONTENT_COLUMN = 1;
    // 数据 MIME 类型列的索引
    private static final int DATA_MIME_TYPE_COLUMN = 2;
    // 数据模式列的索引
    private static final int DATA_MODE_COLUMN = 3;
    // 笔记父 ID 列的索引
    private static final int NOTE_PARENT_ID_COLUMN = 0;
    // 笔记提醒日期列的索引
    private static final int NOTE_ALERTED_DATE_COLUMN = 1;
    // 笔记背景颜色 ID 列的索引
    private static final int NOTE_BG_COLOR_ID_COLUMN = 2;
    // 笔记小部件 ID 列的索引
    private static final int NOTE_WIDGET_ID_COLUMN = 3;
    // 笔记小部件类型列的索引
    private static final int NOTE_WIDGET_TYPE_COLUMN = 4;
    // 笔记修改日期列的索引
    private static final int NOTE_MODIFIED_DATE_COLUMN = 5;

    /**
     * 新建笔记的构造函数
     * @param context 上下文对象
     * @param folderId 文件夹 ID
     */
    private WorkingNote(Context context, long folderId) {
        mContext = context;
        mAlertDate = 0;
        mModifiedDate = System.currentTimeMillis();
        mFolderId = folderId;
        mNote = new Note();
        mNoteId = 0;
        mIsDeleted = false;
        mMode = 0;
        mWidgetType = Notes.TYPE_WIDGET_INVALIDE;
    }

    /**
     * 加载现有笔记的构造函数
     * @param context 上下文对象
     * @param noteId 笔记 ID
     * @param folderId 文件夹 ID
     */
    private WorkingNote(Context context, long noteId, long folderId) {
        mContext = context;
        mNoteId = noteId;
        mFolderId = folderId;
        mIsDeleted = false;
        mNote = new Note();
        loadNote();
    }

    /**
     * 加载笔记的基本信息
     */
    private void loadNote() {
        // 查询笔记的基本信息
        Cursor cursor = mContext.getContentResolver().query(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, mNoteId), NOTE_PROJECTION, null,
                null, null);

        if (cursor != null) {
            if (cursor.moveToFirst()) {
                // 获取笔记的相关信息
                mFolderId = cursor.getLong(NOTE_PARENT_ID_COLUMN);
                mBgColorId = cursor.getInt(NOTE_BG_COLOR_ID_COLUMN);
                mWidgetId = cursor.getInt(NOTE_WIDGET_ID_COLUMN);
                mWidgetType = cursor.getInt(NOTE_WIDGET_TYPE_COLUMN);
                mAlertDate = cursor.getLong(NOTE_ALERTED_DATE_COLUMN);
                mModifiedDate = cursor.getLong(NOTE_MODIFIED_DATE_COLUMN);
            }
            cursor.close();
        } else {
            Log.e(TAG, "No note with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note with id " + mNoteId);
        }
        // 加载笔记的数据
        loadNoteData();
    }

    /**
     * 加载笔记的数据信息
     */
    private void loadNoteData() {
        // 查询笔记的数据信息
        Cursor cursor = mContext.getContentResolver().query(Notes.CONTENT_DATA_URI, DATA_PROJECTION,
                DataColumns.NOTE_ID + "=?", new String[] {
                    String.valueOf(mNoteId)
                }, null);

        if (cursor != null) {
            if (cursor.moveToFirst()) {
                do {
                    // 获取数据的 MIME 类型
                    String type = cursor.getString(DATA_MIME_TYPE_COLUMN);
                    if (DataConstants.NOTE.equals(type)) {
                        // 文本笔记数据
                        mContent = cursor.getString(DATA_CONTENT_COLUMN);
                        mMode = cursor.getInt(DATA_MODE_COLUMN);
                        mNote.setTextDataId(cursor.getLong(DATA_ID_COLUMN));
                    } else if (DataConstants.CALL_NOTE.equals(type)) {
                        // 通话笔记数据
                        mNote.setCallDataId(cursor.getLong(DATA_ID_COLUMN));
                    } else {
                        Log.d(TAG, "Wrong note type with type:" + type);
                    }
                } while (cursor.moveToNext());
            }
            cursor.close();
        } else {
            Log.e(TAG, "No data with id:" + mNoteId);
            throw new IllegalArgumentException("Unable to find note's data with id " + mNoteId);
        }
    }

    /**
     * 创建一个空的笔记
     * @param context 上下文对象
     * @param folderId 文件夹 ID
     * @param widgetId 小部件 ID
     * @param widgetType 小部件类型
     * @param defaultBgColorId 默认背景颜色 ID
     * @return 新创建的 WorkingNote 对象
     */
    public static WorkingNote createEmptyNote(Context context, long folderId, int widgetId,
            int widgetType, int defaultBgColorId) {
        WorkingNote note = new WorkingNote(context, folderId);
        note.setBgColorId(defaultBgColorId);
        note.setWidgetId(widgetId);
        note.setWidgetType(widgetType);
        return note;
    }

    /**
     * 加载一个已有的笔记
     * @param context 上下文对象
     * @param id 笔记 ID
     * @return 加载的 WorkingNote 对象
     */
    public static WorkingNote load(Context context, long id) {
        return new WorkingNote(context, id, 0);
    }

    /**
     * 保存笔记
     * @return 保存是否成功
     */
    public synchronized boolean saveNote() {
        if (isWorthSaving()) {
            if (!existInDatabase()) {
                // 如果笔记不存在于数据库中，创建新的笔记 ID
                if ((mNoteId = Note.getNewNoteId(mContext, mFolderId)) == 0) {
                    Log.e(TAG, "Create new note fail with id:" + mNoteId);
                    return false;
                }
            }
            // 同步笔记数据
            mNote.syncNote(mContext, mNoteId);

            /**
             * 如果笔记关联了小部件，更新小部件的内容
             */
            if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                    && mWidgetType != Notes.TYPE_WIDGET_INVALIDE
                    && mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onWidgetChanged();
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * 判断笔记是否存在于数据库中
     * @return 是否存在
     */
    public boolean existInDatabase() {
        return mNoteId > 0;
    }

    /**
     * 判断笔记是否值得保存
     * @return 是否值得保存
     */
    private boolean isWorthSaving() {
        if (mIsDeleted || (!existInDatabase() && TextUtils.isEmpty(mContent))
                || (existInDatabase() && !mNote.isLocalModified())) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 设置笔记设置变化监听器
     * @param l 监听器对象
     */
    public void setOnSettingStatusChangedListener(NoteSettingChangedListener l) {
        mNoteSettingStatusListener = l;
    }

    /**
     * 设置笔记的提醒日期
     * @param date 提醒日期
     * @param set 是否设置提醒
     */
    public void setAlertDate(long date, boolean set) {
        if (date != mAlertDate) {
            mAlertDate = date;
            mNote.setNoteValue(NoteColumns.ALERTED_DATE, String.valueOf(mAlertDate));
        }
        if (mNoteSettingStatusListener != null) {
            mNoteSettingStatusListener.onClockAlertChanged(date, set);
        }
    }

    /**
     * 标记笔记是否被删除
     * @param mark 是否删除
     */
    public void markDeleted(boolean mark) {
        mIsDeleted = mark;
        if (mWidgetId != AppWidgetManager.INVALID_APPWIDGET_ID
                && mWidgetType != Notes.TYPE_WIDGET_INVALIDE && mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onWidgetChanged();
        }
    }

    /**
     * 设置笔记的背景颜色 ID
     * @param id 背景颜色 ID
     */
    public void setBgColorId(int id) {
        if (id != mBgColorId) {
            mBgColorId = id;
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onBackgroundColorChanged();
            }
            mNote.setNoteValue(NoteColumns.BG_COLOR_ID, String.valueOf(id));
        }
    }

    /**
     * 设置笔记的清单模式
     * @param mode 清单模式
     */
    public void setCheckListMode(int mode) {
        if (mMode != mode) {
            if (mNoteSettingStatusListener != null) {
                mNoteSettingStatusListener.onCheckListModeChanged(mMode, mode);
            }
            mMode = mode;
            mNote.setTextData(TextNote.MODE, String.valueOf(mMode));
        }
    }

    /**
     * 设置笔记关联的小部件类型
     * @param type 小部件类型
     */
    public void setWidgetType(int type) {
        if (type != mWidgetType) {
            mWidgetType = type;
            mNote.setNoteValue(NoteColumns.WIDGET_TYPE, String.valueOf(mWidgetType));
        }
    }

    /**
     * 设置笔记关联的小部件 ID
     * @param id 小部件 ID
     */
    public void setWidgetId(int id) {
        if (id != mWidgetId) {
            mWidgetId = id;
            mNote.setNoteValue(NoteColumns.WIDGET_ID, String.valueOf(mWidgetId));
        }
    }

    /**
     * 设置笔记的文本内容
     * @param text 文本内容
     */
    public void setWorkingText(String text) {
        if (!TextUtils.equals(mContent, text)) {
            mContent = text;
            mNote.setTextData(DataColumns.CONTENT, mContent);
        }
    }

    /**
     * 将笔记转换为通话笔记
     * @param phoneNumber 电话号码
     * @param callDate 通话日期
     */
    public void convertToCallNote(String phoneNumber, long callDate) {
        mNote.setCallData(CallNote.CALL_DATE, String.valueOf(callDate));
        mNote.setCallData(CallNote.PHONE_NUMBER, phoneNumber);
        mNote.setNoteValue(NoteColumns.PARENT_ID, String.valueOf(Notes.ID_CALL_RECORD_FOLDER));
    }

    /**
     * 判断笔记是否设置了提醒
     * @return 是否设置了提醒
     */
    public boolean hasClockAlert() {
        return (mAlertDate > 0 ? true : false);
    }

    /**
     * 获取笔记的内容
     * @return 笔记内容
     */
    public String getContent() {
        return mContent;
    }

    /**
     * 获取笔记的提醒日期
     * @return 提醒日期
     */
    public long getAlertDate() {
        return mAlertDate;
    }

    /**
     * 获取笔记的修改日期
     * @return 修改日期
     */
    public long getModifiedDate() {
        return mModifiedDate;
    }

    /**
     * 获取笔记的背景颜色资源 ID
     * @return 背景颜色资源 ID
     */
    public int getBgColorResId() {
        return NoteBgResources.getNoteBgResource(mBgColorId);
    }

    /**
     * 获取笔记的背景颜色 ID
     * @return 背景颜色 ID
     */
    public int getBgColorId() {
        return mBgColorId;
    }

    /**
     * 获取笔记标题的背景资源 ID
     * @return 标题背景资源 ID
     */
    public int getTitleBgResId() {
        return NoteBgResources.getNoteTitleBgResource(mBgColorId);
    }

    /**
     * 获取笔记的清单模式
     * @return 清单模式
     */
    public int getCheckListMode() {
        return mMode;
    }

    /**
     * 获取笔记的 ID
     * @return 笔记 ID
     */
    public long getNoteId() {
        return mNoteId;
    }

    /**
     * 获取笔记所在的文件夹 ID
     * @return 文件夹 ID
     */
    public long getFolderId() {
        return mFolderId;
    }

    /**
     * 获取笔记关联的小部件 ID
     * @return 小部件 ID
     */
    public int getWidgetId() {
        return mWidgetId;
    }

    /**
     * 获取笔记关联的小部件类型
     * @return 小部件类型
     */
    public int getWidgetType() {
        return mWidgetType;
    }

    /**
     * 笔记设置变化监听器接口
     */
    public interface NoteSettingChangedListener {
        /**
         * 当笔记的背景颜色发生变化时调用
         */
        void onBackgroundColorChanged();

        /**
         * 当用户设置提醒时调用
         * @param date 提醒日期
         * @param set 是否设置提醒
         */
        void onClockAlertChanged(long date, boolean set);

        /**
         * 当用户通过小部件创建笔记时调用
         */
        void onWidgetChanged();

        /**
         * 当笔记的清单模式发生切换时调用
         * @param oldMode 旧模式
         * @param newMode 新模式
         */
        void onCheckListModeChanged(int oldMode, int newMode);
    }
}